# 生成播种所需要的 values;
# v1: 根据地形的变化进行变量播种；
#

class Seeding():
    def __init__(
        self,
        path_prefix,
        uuid,
        name,
        geometry,
        starttime="",  # 2024-
    ):
        pass